﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.IO;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows.DataPresenter.Events;
using System.Collections;

namespace XamTreeView
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        ObservableCollection<Node> source = new ObservableCollection<Node>();
        FieldLayout parent;
        public Window1()
        {
            InitializeComponent();
            DriveInfo drive = new DriveInfo(@"C:\");
            DirectoryInfo root = drive.RootDirectory;

            xamDataGrid1.AssigningFieldLayoutToItem += new EventHandler<AssigningFieldLayoutToItemEventArgs>(xamDataGrid1_AssigningFieldLayoutToItem);    
            xamDataGrid1.FieldLayoutInitialized += new EventHandler<FieldLayoutInitializedEventArgs>(xamDataGrid_FieldLayoutInitialized);
            xamDataGrid1.RecordExpanding += new EventHandler<RecordExpandingEventArgs>(xamDataGrid1_RecordExpanding);
            parent = xamDataGrid1.FieldLayouts[0];
            LoadOnDemand(root, source);

            xamDataGrid1.DataSource = source;
        }

        void xamDataGrid1_AssigningFieldLayoutToItem(object sender, AssigningFieldLayoutToItemEventArgs e)
        {
            // The XDG will reuse the same FieldLayout if the both the parent and nested layout contain the same fields, 
            // so this explicitly sets the nested layout
            if (e.ParentExpandableFieldRecord == null)
                e.FieldLayout = xamDataGrid1.FieldLayouts["master"];
            else
                e.FieldLayout = xamDataGrid1.FieldLayouts["nested"];
        }

        void OnLabelPresenterSizeChanged(object sender, SizeChangedEventArgs e)
        {
            var pres = sender as LabelPresenter;
            if (pres == null )
                return;

            // Ignore tiny changes because they can lead to infinite layout loops.
            double diff = Math.Abs(pres.Field.Settings.LabelWidth - pres.ActualWidth);
            if (diff <= 1)
                return;

            // Set the LabelWidth property so that the LabelWidthResolved is recalculated.
            // That forces the binding to update the width of the corresponding field in 
            // the detail layout.
            pres.Field.Settings.LabelWidth = pres.ActualWidth;
        }

        void xamDataGrid1_RecordExpanding(object sender, Infragistics.Windows.DataPresenter.Events.RecordExpandingEventArgs e)
        {
            try
            {
                if (e.Record.RecordType != Infragistics.Windows.DataPresenter.RecordType.DataRecord)
                    return;

                Node node = (e.Record as DataRecord).DataItem as Node;

                if (node.IsDirectory)
                {
                    DirectoryInfo root = new DirectoryInfo(node.FullName);
                    LoadOnDemand(root, node.ChildNodes);
                }
                else
                {
                    // it's a file
                }
            }
            catch (UnauthorizedAccessException)
            {
                e.Record.ExpansionIndicatorVisibility = Visibility.Hidden;
            }
        }

        void xamDataGrid_FieldLayoutInitialized(object sender, FieldLayoutInitializedEventArgs e)
        {
            FieldLayout nested=null;
            if (parent == e.FieldLayout)
                return;
            else nested = e.FieldLayout;
                // Get all of the visible fields in the master layout.
                List<Field> masterFields =
                    (from f in parent.Fields
                     where f.VisibilityResolved == Visibility.Visible
                     select f)
                    .ToList();

                // Get all of the visible fields in the detail layout.
                List<Field> detailFields =
                    (from f in nested.Fields
                     where f.VisibilityResolved == Visibility.Visible
                     select f)
                    .ToList();

                int iterations = Math.Min(masterFields.Count, detailFields.Count);

                // Bind the width of each field in the detail layout to
                // the resolved/actual width of the corresponding field 
                // in the master layout.
                for (int n = 0; n < iterations; ++n)
                {
                    BindingOperations.SetBinding(
                        detailFields[n].Settings,
                        FieldSettings.LabelWidthProperty,
                        new Binding
                        {
                            Path = new PropertyPath("LabelWidthResolved"),
                            Source = masterFields[n]
                        });
                }
        }

        private void LoadOnDemand(DirectoryInfo root, ObservableCollection<Node> source)
        {
            source.Clear();
            foreach (DirectoryInfo dir in root.GetDirectories())
                source.Add(MakeNodeFromInfo(dir));

            foreach (FileInfo file in root.GetFiles())
                source.Add(MakeNodeFromInfo(file));
        }

        private Node MakeNodeFromInfo(FileSystemInfo info)
        {
            Node node = new Node();
            node.Name = info.Name;
            node.CreationTime = info.CreationTime;
            node.LastAcessTime = info.LastAccessTime;
            node.IsDirectory = info is DirectoryInfo ? true : false;
            node.FullName = info.FullName;

            return node;
        }
    }

    public class Node
    {
        public string Name { get; set; }
        public DateTime CreationTime { get; set; }
        public DateTime LastAcessTime { get; set; }
        public bool IsDirectory { get; set; }
        public ObservableCollection<Node> ChildNodes { get; set; }
        public string FullName { get; set; }
        public Node()
        {
            this.ChildNodes = new ObservableCollection<Node>();
        }
    }
}
